#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import wx
import wx.html as html
import readdic
import re
import string

ID_TEXTEDIT = wx.NewId()
ID_BUTTON = wx.NewId()
ID_LIST = wx.NewId()

projectName = "pydic"
dicfilePath = "/usr/share/pydic/dic/engkor.dic"

#폰트 매트릭스
fontsl = [
    [4, 5, 6, 8, 9, 10, 12],
    [5, 6, 8, 9, 10, 12, 16],
    [6, 8, 9, 10, 12, 16, 20],
    [8, 9, 10, 12, 16, 20, 22],
    [9, 10, 12, 16, 20, 22, 26],
    [10, 12, 16, 20, 22, 26, 30],
    [12, 16, 20, 22, 26, 30, 32]
]

#한글출력을 위한 함수의 간소화
u = lambda str="", encode="utf-8":unicode(str, encode)

class PydicFrame(wx.Frame):
    def __init__(self, *args, **kwds):
	kwds["style"] = wx.DEFAULT_FRAME_STYLE
	wx.Frame.__init__(self, *args, **kwds)

	f = self.GetFont()

        self.teInput = wx.TextCtrl(self, ID_TEXTEDIT, "", style=wx.TE_PROCESS_ENTER)
        self.bInput = wx.Button(self, ID_BUTTON, u("찾기"))
        self.lbWordlist = wx.ListBox(self, ID_LIST, choices=[], style=wx.LB_SINGLE)
        self.htmlMeaning = html.HtmlWindow(self, -1)
        self.htmlMeaning.SetFonts(f.GetFaceName(), f.GetFaceName(), fontsl[2])

	self.wordDic = {}
	self.wordList = []
        self.wordPos = 0

        self.__set_properties()
        self.__do_layout()

	self.Bind(wx.EVT_TEXT, self.OnText, self.teInput)
	self.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter, self.teInput)
        self.Bind(wx.EVT_LISTBOX, self.OnSelect, self.lbWordlist)
        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

	self.dicFile = self.openDic(dicfilePath)
        self.setWordlist()
       
	self.teInput.SetFocus()

    def __set_properties(self):
        self.SetTitle(u(projectName))
        self.SetSize((400, 300))

    def __do_layout(self):
        vboxMain = wx.BoxSizer(wx.VERTICAL)
        hboxContent = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop = wx.BoxSizer(wx.HORIZONTAL)
        hboxTop.Add(self.teInput, 1, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 1)
        hboxTop.Add(self.bInput, 0, wx.FIXED_MINSIZE, 0)
        vboxMain.Add(hboxTop, 0, wx.EXPAND, 0)
        hboxContent.Add(self.lbWordlist, 0, wx.EXPAND, 0)
        hboxContent.Add(self.htmlMeaning, 1, wx.EXPAND, 0)
        vboxMain.Add(hboxContent, 1, wx.EXPAND, 0)
        self.SetAutoLayout(1)
        self.SetSizer(vboxMain)
        self.Layout()
        self.Centre()

    def openDic(self, dicPath):
        dicFile = open(dicPath, "r")

        if readdic.verifyDic(dicFile, "LDIC"):
            pass
        else:
            print u("올바른 사전 파일이 아닙니다.")
            exit

        list, l_offset_s, l_offset_e, mean, m_offset_s, m_offset_e, fnt, f_offset_s, f_offset_e = readdic.readHeader(dicFile, "cllcllcll")
        self.wordDic = readdic.makeWordlist(0, dicFile, l_offset_e)
        self.m_offset_s = m_offset_s

        self.wordList = self.wordDic.keys()
        self.wordList.sort()

        return dicFile

    def OnText(self, e):
        word = self.teInput.GetValue()
        self.search(word)

    def OnTextEnter(self, e):
        self.teInput.SetSelection(0, -1. -1) #FIXME:스팩상 인자값은 두개임
        self.setWordlist()

    def OnSelect(self, e):
        selectedItem = self.lbWordlist.GetStringSelection()
        self.search(selectedItem)
        self.teInput.SetFocus()
        self.teInput.SetValue(selectedItem)

    def OnResize(self, e):
        self.Layout() #윈도우 사이즈가 바뀌면 자동으로 호출되는 함수
        self.setWordlist()

    def OnCloseWindow(self, e):
        self.Destroy()

    def setWordlist(self):
        lbWidth, lbHeight = self.lbWordlist.GetClientSizeTuple()
        lbCharHeight = self.lbWordlist.GetCharHeight()

        numWord = (lbHeight / (float(lbCharHeight) + 2.2)) #FIXME:Hardcoding
        numWord = int(numWord)

        if (self.wordPos < (numWord / 2)):
            startPos = 0
            endPos = numWord
            wordPos = self.wordPos
        else:
            if (numWord % 2):
                startPos = self.wordPos - (numWord / 2)
                endPos = self.wordPos + (numWord / 2)
                wordPos = endPos - startPos - (numWord / 2)

            else:
                startPos = self.wordPos - (numWord / 2) + 1
                endPos = self.wordPos + (numWord / 2)
                wordPos = endPos - startPos - (numWord / 2)

        temp = self.wordList[startPos:endPos + 1]
        self.lbWordlist.Set(temp)
        self.lbWordlist.SetSelection(wordPos)

    def search(self, word): #FIXME:인코딩 관련
        try:
            self.wordPos = self.wordList.index(word)

            meaning = unicode(readdic.searchWord(self.dicFile, self.wordDic[word], self.m_offset_s), "euc-kr")
            temp = string.replace(meaning, "./font/", "/usr/share/pydic/font/");
            self.htmlMeaning.SetPage(temp)
        except:
            pass

class Pydic(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        pydicFrame = PydicFrame(None, -1, "")
        self.SetTopWindow(pydicFrame)
        pydicFrame.Show(True)
        return 1

#if __name__ == "__main__":
def Main():
    try:
        pydic = Pydic()
        pydic.MainLoop()
    except:
        import sys, traceback
        xc = traceback.format_exception(*sys.exc_info())
        wx.MessageBox(u(''.join(xc)))
